# CSPP Geo Gridded GLM v1.0 Beta1

CSPP Geo Gridded GLM software takes GOES GLM Level 2+ LCFA files and grids them to the ABI fixed grid, creating a new set of GLM gridded products which were developed by the GOES-R GLM Science Team. Tiles can optionally be created in an AWIPS-compatible format.

The new gridded products are:

- Minimum Flash Area
- Flash Extent Density
- Total Optical Energy

This software was developed at the University of Wisconsin - Madison, SSEC / CIMSS, under funding provided by the NOAA STAR / GOES-R Program.

The latest version of this software can be obtained from: https://cimss.ssec.wisc.edu/csppgeo/

Questions and comments can be directed to csppgeo.issues@ssec.wisc.edu.

## ACKNOWLEDGEMENTS

The developers wish to thank:

- Eric Bruning, developer of the open source glmtools package which this software is based on,
- Scott Rudlosky, GOES-R GLM Science Team lead and PI of the NOAA GLM Gridded Products effort,
- Lee Byerle and Joe Zajic, NWS TOWR-S team, for their assistance with tile development and AWIPS compatibility.

## COPYRIGHT / LICENSE / DISCLAIMER

Portions of the source code in this software package are: 

Copyright (C) 2020-2021 Space Science and Engineering Center (SSEC), University of Wisconsin-Madison.

Original source code, scripts and automation included as part of this package are distributed under the GNU General Public License agreement version 3. 

Binary executable files and third-party source code included as part of this software package are copyrighted and licensed by their respective organizations, and distributed consistent with their licensing terms.

The University of Wisconsin-Madison Space Science and Engineering Center (SSEC) makes no warranty of any kind with regard to the CSPP software or any accompanying documentation, including but not limited to the implied warranties of merchantability and fitness for a particular purpose. SSEC does not indemnify any infringement of copyright, patent, or trademark through the use or modification of this software. All risk of use is assumed by the user. Users agree not to hold SSEC, the University of Wisconsin-Madison, or any of its employees or assigns liable for any consequences resulting from the use of the CSPP software.

## INSTALLATION

To install, extract the tarball and optionally add the bin directory to your PATH.

```
tar xf cspp-geo-gridded-glm-1.0b1.tar.xz
export PATH=$PATH:$PWD/cspp-geo-gridded-glm-1.0b1/bin
```

A test data tarball can be downloaded from the same location as the software. Reference output is included. Refer to the EXAMPLES section below to run the test case.​

## USAGE

**cspp-geo-gglm.sh**

The main processing script creates one minute grids from GLM L2+ LCFA input files, which can be obtained from the CSPP Geo GRB software or from NOAA CLASS. It is recommended that all 3 20-second input files from the same minute be specified, to ensure that the gridded product captures all lightning activity from that minute. Alternatively the script can be run in "realtime" mode where, provided a single input file, it will determine if an entire minute is available and if so begin gridding. 

For a list of options, run `cspp-geo-gglm.sh -h`.


**cspp-geo-gglm-make-grids.sh**

This is a wrapper for the `make_GLM_grids.py` example utility provided by glmtools. Users may
find this script useful for creating grids with custom resolutions or frame durations.

## OUTPUT

The Gridded GLM product is written by the main processing script in the form of NetCDF4 files spanning a clock-aligned minute of input data. The file naming convention and file format are modeled after the GOES-R operational format, as documented in the GOES-R Product Definition and Users' Guide (PUG).

Example Gridded GLM product filename: `CG_GLM-L2-GLMF-M3_G16_s20192991759000_e20192991800000_c20210492345130.nc`

A set of tiles can optionally be created for each Gridded GLM file. The tiles are also NetCDF4 files and designed to be compatible with AWIPS. Tiles covering geographic areas without lightning activity are omitted. The time in the filename is the observation end time.

Example tile filename: `CSPP_OR_GLM-L2-GLMF-M3_G16_T15_20191026180000.nc`


## EXAMPLES

Create a single one-minute gridded product file by specifying three input 20-second GLM L2+ files

`cspp-geo-gglm.sh /data/GLM-L2/2019299/17/OR_GLM-L2-LCFA_G16_s20192991759*`

Create a single one-minute gridded product file by specifying only one of the three input 20-second GLM L2+ files ("realtime" mode)

`cspp-geo-gglm.sh -1 /data/GLM-L2/2019299/17/OR_GLM-L2-LCFA_G16_s20192991759400_e20192991800000_c20192991800031.nc`

Create a gridded product file and tiles

`cspp-geo-gglm.sh -1 -t /data/GLM-L2/2019299/17/OR_GLM-L2-LCFA_G16_s20192991759400_e20192991800000_c20192991800031.nc`


## CAVEATS

The following issues are known to affect this release

- The DQF (data quality flags) are always set to 0 in this version. Proper data quality information will be added in a future version
- In the default processing mode (not "realtime" mode), if the first 20-second file of the minute is not specified, the output file time will be offset from the nominal minute boundaries. This will be addressed in a future release, but note that it is recommended to provide all 3 20-second input files, or to run in "realtime mode" (option `-1`).
- Tile generation with CONUS and mesoscale gridding has not been fully tested
- The secondary script "cspp-geo-gglm-make-grids.sh" has not been fully tested


## REFERENCES

Documentation:

- [GLM Quick Guides](https://www.star.nesdis.noaa.gov/GOES/documents/GLM_Quick_Guides_May_2019.pdf)
- [GLM Full Disk Quick Guide](http://rammb.cira.colostate.edu/training/visit/quick_guides/GLM_Full_Disk_Quick_Guide_V1.pdf)
- [GLM Minimum Flash Area Quick Guide](http://rammb.cira.colostate.edu/training/visit/quick_guides/GLM_MFA_Quick_Guide_V1.pdf)
- [GOES-R Lightning Detection Fact Sheet](https://www.goes-r.gov/education/docs/fs_lightning.pdf)
- [GOES-R Product Definition and Users' Guide](https://www.goes-r.gov/resources/docs.html)

Software:

- [GLMTools software repository](https://github.com/deeplycloudy/glmtools)

Papers:

Bruning, E., Tillier, C. E., Edgington, S. F., Rudlosky, S. D., Zajic, J.,Gravelle, C., et al.
(2019). Meteorological imagery for the Geostationary Lightning Mapper. Journal of Geophysical
Research: Atmospheres, 2019; 124: 14285 14309. https://doi.org/10.1029/2019JD030874
